@@ -15,6 +15,9 @@ class TSVMEncoder(DictionaryEncoder[LT]):
             return np.array([-1])
         return super().encode(label_list)
 
+    def initialize(self, labels: Iterable[LT]) -> None:
+        return super().initialize(labels)
+
     
 class TSVM(il.SkLearnVectorClassifier[IT, KT, LT], Generic[IT, KT, LT]):
     def __init__(self, 
@@ -22,4 +25,7 @@ class TSVM(il.SkLearnVectorClassifier[IT, KT, LT], Generic[IT, KT, LT]):
                  filename: "Optional[PathLike[str]]" = None) -> None:
         estimator = SKTSVM()
         encoder = TSVMEncoder.from_list([])
-        super().__init__(estimator, encoder, storage_location, filename)
\ No newline at end of file
+        super().__init__(estimator, encoder, storage_location, filename)
+
+    def _filter_x_only_encoded_y(self, instances: Iterable[_T], labelings: Sequence[Iterable[LT]]) -> Tuple[Iterable[_T], np.ndarray]:
+        return super()._filter_x_only_encoded_y(instances, labelings)
\ No newline at end of file
